﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Collections.Generic;
using System.Web.Mvc;
using ApplicationManagement.Contracts.BusinessLogic;
using ApplicationManagement.Models;
using ApplicationManagement.Web.Helpers;
using Newtonsoft.Json;
using ApplicationManagement.Web.Models;


namespace ApplicationManagement.Web.Controllers
{
     [OutputCache(NoStore = true, Duration = 0, VaryByParam = "*")]
    public class ApplicationController : Controller
    {
        //
        // GET: /Application/
         private readonly IApplicationBusinessLogic<Application> _applicationlogic;
         private readonly IDataCarrierBusinessLogic<DataCarrier> _datacarrierlogic;
         private readonly IApplicationDataCarrierBusinessLogic<ApplicationDataCarrier> _applicationdatacarrierlogic;
         private readonly IApplicationTechniqueBusinessLogic<ApplicationTechnique> _applicationtechniquelogic;
         private readonly ITechniqueBusinessLogic<Technique> _techniquelogic;
         
        //
        // GET: /ApplicationType/
         
         public ApplicationController(IApplicationBusinessLogic<Application> applicationlogic, IDataCarrierBusinessLogic<DataCarrier> datacarrierlogic, IApplicationDataCarrierBusinessLogic<ApplicationDataCarrier> applicationdatacarrierlogic, IApplicationTechniqueBusinessLogic<ApplicationTechnique> applicationtechniquelogic, ITechniqueBusinessLogic<Technique> techniquelogic)
        {
            _applicationlogic = applicationlogic;
            _datacarrierlogic = datacarrierlogic;
            _applicationdatacarrierlogic = applicationdatacarrierlogic;
            _applicationtechniquelogic = applicationtechniquelogic;
            _techniquelogic = techniquelogic;
           
        }
         public ActionResult ShowProductData()
         {
             return View("ApplicationTypeController");
         }
         [ExceptionHelper]
         public JsonResult GetApplication(string ApplicationFilter)
         {
             string innerError = string.Empty;
             bool successful = true;
             List<Application> application = null;
             try
             {
                 application = _applicationlogic.GetApplication(ApplicationFilter);
             }
             catch (Exception ex)
             {
                 innerError = ex.Message;
                 successful = false;
             }
             Debug.Assert(application != null, "application != null");
             var collection = application.Select(x => new
             {
                x.ApplicationId,
                x.Name,
               // x.LifeCycleId,
                x.SourceCodeLocation,
                x.InhouseDeveloped,                
                x.InstallationMethodId,
                x.ApplicationTypeId,
                x.SupplierId,
                x.SystemId,
                x.StartOfLife,
                x.StartOfOperation,
                x.EndOfMainstream,
                x.EndOfContainment,
                x.EndOfSupport,
                x.EndOfLife,
                x.ChangedBy,
                x.Timestamp,
                x.Comment                
             });
             return Json(new { success = successful, allApplication = collection });
         }

         [ExceptionHelper]
         public JsonResult CreateApplication(Application application)
         {
             string innerError = string.Empty;
             bool successful = true;
             try
             {
                 string username = null;
                 username = Request.ServerVariables["LOGON_USER"];
                 application.ChangedBy = username;
                 
                 string s_message = Helpers.DateCheckingHelper.checkDate(application.StartOfLife, application.StartOfOperation, application.EndOfMainstream, application.EndOfContainment, application.EndOfSupport, application.EndOfLife);
                 if (s_message.Length > 0)
                 {
                     return Json(new
                     {
                         success = false,
                         innererror = s_message
                     });
                 }
                 _applicationlogic.AddApplication(application);
             }
             catch (Exception ex)
             {
                 innerError = ex.Message;
                 successful = false;
             }
             return Json(new
             {
                 success = successful,
                 innererror = innerError
             });
         }
         [ExceptionHelper]
         public JsonResult DeleteApplication(Application application)
         {
             string innerError = string.Empty;
             bool successful = true;
             try
             {
                 // Delete the Application datacarrier from middel table.
                 var applicationdatacarrierdelete = _applicationdatacarrierlogic.GetApplicationdatacarrierbyappid(application.ApplicationId).ToList();
                 foreach (var applicationdatacarrierdeleted in applicationdatacarrierdelete)
                 {
                     ApplicationDataCarrier applicationdatacarrier = new ApplicationDataCarrier();
                     applicationdatacarrier.ApplicationDataCarrierId = applicationdatacarrierdeleted.ApplicationDataCarrierId;
                     applicationdatacarrier.DataCarrierId = applicationdatacarrierdeleted.DataCarrierId;
                     applicationdatacarrier.ApplicationId = applicationdatacarrierdeleted.ApplicationId;
                     applicationdatacarrier.ChangedBy = applicationdatacarrierdeleted.ChangedBy;
                     applicationdatacarrier.Timestamp = applicationdatacarrierdeleted.Timestamp;
                     _applicationdatacarrierlogic.DeleteApplicationDataCarrier(applicationdatacarrier);
                 }
                 // Delete the Application Technique from middel table.
                 var applicationtechniquedelet  = _applicationtechniquelogic.GetApplicationtechniquebyappid(application.ApplicationId).ToList();
                 foreach (var applicationdatacarrierdeleted in applicationtechniquedelet)
                 {
                     ApplicationTechnique applicationetechnique = new ApplicationTechnique();
                     applicationetechnique.ApplicationTechniqueId = applicationdatacarrierdeleted.ApplicationTechniqueId;
                     applicationetechnique.TechniqueId = applicationdatacarrierdeleted.TechniqueId;
                     applicationetechnique.ApplicationId = applicationdatacarrierdeleted.ApplicationId;
                     applicationetechnique.ChangedBy = applicationdatacarrierdeleted.ChangedBy;
                     applicationetechnique.Timestamp = applicationdatacarrierdeleted.Timestamp;
                     _applicationtechniquelogic.DeleteApplicationTechnique(applicationetechnique);
                 }
                 _applicationlogic.Deleteapplication(application);
             }
             catch (Exception ex)
             {
                 innerError = ex.Message;
                 successful = false;
             }
             return Json(new
             {
                 success = successful,
                 innererror = innerError
             });
         }
         [ExceptionHelper]
         public JsonResult EditApplication(Application application)
         {
             string innerError = string.Empty;
             bool successful = true;
             try
             {               
                 string s_message = Helpers.DateCheckingHelper.checkDate(application.StartOfLife, application.StartOfOperation, application.EndOfMainstream, application.EndOfContainment, application.EndOfSupport, application.EndOfLife);
                 if (s_message.Length > 0)
                 {
                     return Json(new
                     {
                         success = false,
                         innererror = s_message
                     });
                 }
                 application.Timestamp = DateTime.Now ;
                 application.ChangedBy = Request.ServerVariables["LOGON_USER"];
                 _applicationlogic.Editapplication(application);
             }
             catch (Exception ex)
             {
                 innerError = ex.Message;
                 successful = false;
             }
             return Json(new
             {
                 success = successful,
                 innererror = innerError
             });
         }
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult ShowPopUpDatacarrier(int? appid)
        {
            // To get The Middel Table Data for the Selected Application data Carrier.
            List<ApplicationDataCarrier> applciationdatacarrierlist = new List<ApplicationDataCarrier>();
            var applicationdatacarrierResultnew = _applicationdatacarrierlogic .GetApplicationdatacarrierbyappid (appid).ToList ();
            foreach (var applicationdatacarrier in applicationdatacarrierResultnew)
            {
                applciationdatacarrierlist.Add(new ApplicationDataCarrier { DataCarrier = applicationdatacarrier.DataCarrier, ApplicationId = applicationdatacarrier.ApplicationId, DataCarrierId = applicationdatacarrier.DataCarrierId  });
            }

            // To get The data carrier Table data.
            List<DataCarrier> datacarriernew = new List<DataCarrier>();
            var datacarriernew_list = _datacarrierlogic.GetAllDataCarriers().ToList();
            
            foreach (var datacarrier in datacarriernew_list)
            {
                datacarriernew.Add(new DataCarrier { Description = datacarrier.Description, DataCarrierId = datacarrier.DataCarrierId });                
            }
            datacarriernew.RemoveAll(x => applciationdatacarrierlist.Exists (w => w.DataCarrierId == x.DataCarrierId ));
            
            // To get The Application table Data.
            List<Application> applicationlist = new List<Application>();
            var applicationResult = _applicationlogic.GetApplicationByID(appid).ToList();
            foreach (var application in applicationResult)
            {
                applicationlist.Add(new Application { ApplicationId = application.ApplicationId , Name = application .Name });
            }

            //To get The Middel Table Data for the Selected Application technique.
            List<ApplicationTechnique> applicationtechniquelist = new List<ApplicationTechnique>();
            var applicationtechniqueResultnew = _applicationtechniquelogic.GetApplicationtechniquebyappid(appid).ToList();
            foreach (var applicationtechnique in applicationtechniqueResultnew)
            {
                applicationtechniquelist.Add(new ApplicationTechnique { Technique =  applicationtechnique.Technique , TechniqueId = applicationtechnique.TechniqueId });
            }

            // To get the technique Data
            List<Technique> techniquelist = new List<Technique>();
            var techniquenew_list = _techniquelogic.GetAllTechnique().ToList();
            foreach (var technique in techniquenew_list)
            {
                techniquelist.Add(new Technique { Name = technique.Name , TechniqueId = technique.TechniqueId });  
            }
            techniquelist.RemoveAll(x => applicationtechniquelist.Exists(w => w.TechniqueId == x.TechniqueId));


            if (appid == null)
            {
                appid = 0;
            }
            ApplicationDataCarrierViewModel applicationviewmodel = new ApplicationDataCarrierViewModel()
                {
                    ApplicationDataCarriers = applciationdatacarrierlist,
                    DataCarriers = datacarriernew,
                    Applications = applicationlist,
                    ApplicatationID = (Int32)appid,
                    Techniques = techniquelist,
                    ApplicationTechniques = applicationtechniquelist,
                };
                return View(applicationviewmodel);         
        }
        [ExceptionHelper]
        public JsonResult SaveDataCarriers(int[] datacarriers, int[] deleteditem, int ApplicationID)
        {
            string innerError = string.Empty;
            bool successful = true;
            int ApplicationDataID ; 
            // Adding the Record 
            foreach (var element in datacarriers)
            {
                List<ApplicationDataCarrier> application_datacarrier = null;

                application_datacarrier = _applicationdatacarrierlogic.GetApplicationdatacarrierbydatacarrierID(element, ApplicationID);
                if (application_datacarrier.Count == 0)
                { 
                  ApplicationDataCarrier application_data =  new ApplicationDataCarrier();
                  application_data.ApplicationId = (Int32)ApplicationID;
                  application_data.DataCarrierId = (Int32)element;
                  application_data.ChangedBy = Request.ServerVariables["LOGON_USER"];
                  _applicationdatacarrierlogic.AddApplicationDataCarrier(application_data);
                }
            }   
            // Deleting The Recrod
            if (deleteditem != null)
            {
                foreach (var delete_element in deleteditem)
                {
                    List<ApplicationDataCarrier> application_datacarrier = null;
                    application_datacarrier = _applicationdatacarrierlogic.GetApplicationdatacarrierbydatacarrierID((Int32)delete_element, (Int32)ApplicationID);
                    if (application_datacarrier.Count != 0)
                    {
                        ApplicationDataID = application_datacarrier.Select (x =>x.ApplicationDataCarrierId ).Single();
                        ApplicationDataCarrier application_data = new ApplicationDataCarrier();
                        application_data.ApplicationDataCarrierId = ApplicationDataID; 
                        application_data.ApplicationId = (Int32)ApplicationID;
                        application_data.DataCarrierId = (Int32)delete_element;
                        application_data.ChangedBy = Request.ServerVariables["LOGON_USER"];                        
                        _applicationdatacarrierlogic.DeleteApplicationDataCarrier(application_data);
                    }
                }
            }        
            return Json(new
            {
                success = successful,
                innererror = innerError
            });
        }
        [ExceptionHelper]
        public JsonResult Savetechnique(int[] techniques, int[] deleteditem, int ApplicationID)
        {
            string innerError = string.Empty;
            bool successful = true;
            int ApplicationDataID;
            // Adding the Record 
            foreach (var element in techniques)
            {
                List<ApplicationTechnique> application_technique = null;

                application_technique =_applicationtechniquelogic.GetApplicationtechniquebytechniqueID(element, ApplicationID);
                if (application_technique.Count == 0)
                {
                    ApplicationTechnique application_tech_data = new ApplicationTechnique();
                    application_tech_data.ApplicationId = (Int32)ApplicationID;
                    application_tech_data.TechniqueId = (Int32)element;
                    application_tech_data.ChangedBy = Request.ServerVariables["LOGON_USER"];
                    _applicationtechniquelogic.AddApplicationTechnique(application_tech_data);                    
                }
            }
            // Deleting The Recrod
            if (deleteditem != null)
            {
                foreach (var delete_element in deleteditem)
                {
                    List<ApplicationTechnique> application_technique = null;
                    application_technique = _applicationtechniquelogic.GetApplicationtechniquebytechniqueID((Int32)delete_element, ApplicationID);
                    if (application_technique.Count != 0)
                    {
                        ApplicationDataID = application_technique.Select(x => x.ApplicationTechniqueId ).Single();
                        ApplicationTechnique application_tech_data = new ApplicationTechnique();
                        application_tech_data.ApplicationTechniqueId = ApplicationDataID;
                        application_tech_data.ApplicationId = (Int32)ApplicationID;
                        application_tech_data.TechniqueId = (Int32)delete_element;
                        application_tech_data.ChangedBy = Request.ServerVariables["LOGON_USER"];
                        _applicationtechniquelogic.DeleteApplicationTechnique(application_tech_data);                         
                    }
                }
            }
            return Json(new
            {
                success = successful,
                innererror = innerError
            });
        }
               
            
    }
}
